#include "stdafx.h"
#include "UIInventoryWnd.h"

#include "xrXMLParser.h"
#include "UIXmlInit.h"
#include "../string_table.h"

#include "../actor.h"
#include "../uigamesp.h"
#include "../hudmanager.h"

#include "../CustomOutfit.h"

#include "../weapon.h"

#include "../script_space.h"
#include "../script_process.h"

#include "../eatable_item.h"
#include "../inventory.h"

#include "UIInventoryUtilities.h"
using namespace InventoryUtilities;


#include "../InfoPortion.h"
#include "../level.h"
#include "../game_base_space.h"
#include "../entitycondition.h"

#include "../game_cl_base.h"
#include "UISleepWnd.h"
#include "../ActorCondition.h"
#include "UIDragDropListEx.h"
#include "UIOutfitSlot.h"

#define MAX_ITEMS	70

#define				INVENTORY_ITEM_XML		"inventory_item.xml"
#define				INVENTORY_XML			"inventory_new.xml"
#define				INVENTORY_CHARACTER_XML	"inventory_character.xml"



CUIInventoryWnd*	g_pInvWnd = NULL;

CUIInventoryWnd::CUIInventoryWnd()
{
	m_iCurrentActiveSlot				= NO_ACTIVE_SLOT;
	Hide								();	

	UISellAll							= NULL;

	Init								();
	SetCurrentItem						(NULL);

	SetFont								(HUD().Font().pFontMedium);
	g_pInvWnd							= this;	
	m_b_need_reinit						= false;
}

void CUIInventoryWnd::Init()
{
	CUIXml								uiXml;
	bool xml_result						= uiXml.Init(CONFIG_PATH, UI_PATH, INVENTORY_XML);
	R_ASSERT3							(xml_result, "xml file not found", INVENTORY_XML);

	CUIXmlInit							xml_init;
	CStringTable						string_table;

	CUIWindow::Init						(CUIXmlInit::ApplyAlignX(0, alCenter),
										CUIXmlInit::ApplyAlignY(0, alCenter),
										UI_BASE_WIDTH, UI_BASE_HEIGHT);

	AttachChild							(&UIStaticTop);
	UIStaticTop.Init					("ui\\ui_inv_quick_slots", 0,0,UI_BASE_WIDTH,128);
	AttachChild							(&UIStaticBottom);
	UIStaticBottom.Init					("ui\\ui_bottom_background", 0, UI_BASE_HEIGHT-32, UI_BASE_WIDTH,32);

	AttachChild							(&UIStaticBelt);
	xml_init.InitStatic					(uiXml, "static", 0, &UIStaticBelt);
	
	AttachChild							(&UIBagWnd);
	xml_init.InitStatic					(uiXml, "bag_static", 0, &UIBagWnd);
	
	UIBagWnd.AttachChild				(&UIMoneyWnd);
	xml_init.InitStatic					(uiXml, "money_static", 0, &UIMoneyWnd);


	AttachChild							(&UIDescrWnd);
	xml_init.InitStatic					(uiXml, "descr_static", 0, &UIDescrWnd);

	UIDescrWnd.AttachChild				(&UIItemInfo);
	UIItemInfo.Init						(0, 0, UIDescrWnd.GetWidth(), UIDescrWnd.GetHeight(), INVENTORY_ITEM_XML);

	UIDescrWnd.AttachChild				(&UIDropButton);
	xml_init.InitButton					(uiXml, "drop_button", 0, &UIDropButton);
	UIDropButton.SetMessageTarget		(this);

	AttachChild							(&UIPersonalWnd);
	xml_init.InitFrameWindow			(uiXml, "character_frame_window", 0, &UIPersonalWnd);

	// 
	UIPersonalWnd.AttachChild			(&UIProgressBarHealth);
	xml_init.InitProgressBar			(uiXml, "progress_bar_health", 0, &UIProgressBarHealth);
	
	UIPersonalWnd.AttachChild			(&UIProgressBarSatiety);
	xml_init.InitProgressBar			(uiXml, "progress_bar_satiety", 0, &UIProgressBarSatiety);

	UIPersonalWnd.AttachChild			(&UIProgressBarPsyHealth);
	xml_init.InitProgressBar			(uiXml, "progress_bar_psy", 0, &UIProgressBarPsyHealth);

	UIPersonalWnd.AttachChild			(&UIProgressBarRadiation);
	xml_init.InitProgressBar			(uiXml, "progress_bar_radiation", 0, &UIProgressBarRadiation);

	UIPersonalWnd.AttachChild			(&UIStaticPersonal);
	UIStaticPersonal.Init				("ui\\ui_inv_personal_over_b", -1, UIPersonalWnd.GetHeight() - 175, 260, 260);

	//  
	// attributs suit of character (actor)

	UIStaticPersonal.AttachChild		(&UIOutfitInfo);
	xml_init.InitStatic					(uiXml, "outfit_info_window",0, &UIOutfitInfo);
	UIOutfitInfo.SetText				(string_table);

	//  
	xml_init.InitAutoStatic				(uiXml, "auto_static", this);


	if (GameID() == GAME_SINGLE){
		UISleepWnd						= xr_new<CUISleepWnd>();UISleepWnd->SetAutoDelete(true);
		UISleepWnd->Init				();
		xml_init.InitStatic				(uiXml, "sleep_window", 0, UISleepWnd);
		AttachChild						(UISleepWnd);
	}
	else if(GameID() == GAME_ARTEFACTHUNT){
		UISellAll = xr_new<CUI3tButton>	(); UISellAll->SetAutoDelete(true);
		CUIXmlInit::Init3tButton		(uiXml, "btn_sell_all", 0, UISellAll);
		AttachChild						(UISellAll);
	}

	m_pUIBagList						= xr_new<CUIDragDropListEx>(); UIBagWnd.AttachChild(m_pUIBagList); m_pUIBagList->SetAutoDelete(true);
	xml_init.InitDragDropListEx			(uiXml, "dragdrop_bag", 0, m_pUIBagList);
	BindDragDropListEnents				(m_pUIBagList);

	m_pUIBeltList						= xr_new<CUIDragDropListEx>(); AttachChild(m_pUIBeltList); m_pUIBeltList->SetAutoDelete(true);
	xml_init.InitDragDropListEx			(uiXml, "dragdrop_belt", 0, m_pUIBeltList);
	BindDragDropListEnents				(m_pUIBeltList);

	m_pUIOutfitList						= xr_new<CUIOutfitDragDropList>(); AttachChild(m_pUIOutfitList); m_pUIOutfitList->SetAutoDelete(true);
	xml_init.InitDragDropListEx			(uiXml, "dragdrop_outfit", 0, m_pUIOutfitList);
	BindDragDropListEnents				(m_pUIOutfitList);

	string128	str;
	for(int i=0; i<SLOTS_NUM; ++i){
		m_pUITopList[i]					= xr_new<CUIDragDropListEx>(); AttachChild(m_pUITopList[i]); m_pUITopList[i]->SetAutoDelete(true);
		sprintf(str,"dragdrop_%d",i);
		xml_init.InitDragDropListEx		(uiXml, str, 0, m_pUITopList[i]);
		BindDragDropListEnents			(m_pUITopList[i]);
	}

	//pop-up menu
	AttachChild							(&UIPropertiesBox);
	UIPropertiesBox.Init				("ui_pop_up",0,0,300,300);
	UIPropertiesBox.Hide				();

	AttachChild							(&UITimeWnd);
	xml_init.InitStatic					(uiXml, "time_static", 0, &UITimeWnd);

	AttachChild							(&UIExitButton);
	xml_init.InitButton					(uiXml, "exit_button", 0, &UIExitButton);
	
//Load sounds

	XML_NODE* stored_root				= uiXml.GetLocalRoot		();
	uiXml.SetLocalRoot					(uiXml.NavigateToNode		("action_sounds",0));
	::Sound->create						(sounds[eInvSndOpen],		true, uiXml.Read("snd_open",			0,	NULL));
	::Sound->create						(sounds[eInvSndClose],		true, uiXml.Read("snd_close",			0,	NULL));
	::Sound->create						(sounds[eInvItemToSlot],	true, uiXml.Read("snd_item_to_slot",	0,	NULL));
	::Sound->create						(sounds[eInvItemToBelt],	true, uiXml.Read("snd_item_to_belt",	0,	NULL));
	::Sound->create						(sounds[eInvItemToRuck],	true, uiXml.Read("snd_item_to_ruck",	0,	NULL));
	::Sound->create						(sounds[eInvProperties],	true, uiXml.Read("snd_properties",		0,	NULL));
	::Sound->create						(sounds[eInvDropItem],		true, uiXml.Read("snd_drop_item",		0,	NULL));
	::Sound->create						(sounds[eInvAttachAddon],	true, uiXml.Read("snd_attach_addon",	0,	NULL));
	::Sound->create						(sounds[eInvDetachAddon],	true, uiXml.Read("snd_detach_addon",	0,	NULL));

	uiXml.SetLocalRoot					(stored_root);
}

CUIInventoryWnd::EListType CUIInventoryWnd::GetType(CUIDragDropListEx* l)
{
	if(l==m_pUIBagList)		return iwBag;
	if(l==m_pUIBeltList)	return iwBelt;
	VERIFY					(l==m_pUITopList[0] || l==m_pUITopList[1] || l==m_pUITopList[2] || l==m_pUITopList[3] || l==m_pUITopList[4] || l==m_pUIOutfitList);
	return					iwSlot;
}

void CUIInventoryWnd::PlaySnd(eInventorySndAction a)
{
	if (sounds[a]._handle())
        sounds[a].play					(NULL, sm_2D);
}

CUIInventoryWnd::~CUIInventoryWnd()
{
//.	ClearDragDrop(m_vDragDropItems);
	ClearAllLists						();
}

bool CUIInventoryWnd::OnMouse(float x, float y, EUIMessages mouse_action)
{
	//     
	if(mouse_action == WINDOW_RBUTTON_DOWN)
	{
		if(UIPropertiesBox.IsShown())
		{
			UIPropertiesBox.Hide		();
			return						true;
		}
	}

	CUIWindow::OnMouse					(x, y, mouse_action);

	return true; // always returns true, because ::StopAnyMove() == true;
}

void CUIInventoryWnd::Draw()
{
	CUIWindow::Draw						();
}


void CUIInventoryWnd::Update()
{
	if(m_b_need_reinit)
		InitInventory					();


	CEntityAlive *pEntityAlive			= smart_cast<CEntityAlive*>(Level().CurrentEntity());

	if(pEntityAlive) 
	{
		UIProgressBarHealth.SetProgressPos(s16(pEntityAlive->conditions().GetHealth()*1000));
		if (IsGameTypeSingle())
			UIProgressBarSatiety.SetProgressPos(s16(pEntityAlive->conditions().GetSatiety()*1000));
		else
		{
			if (Level().CurrentViewEntity())
			{
				game_PlayerState* ps	= Game().GetPlayerByGameID(Level().CurrentViewEntity()->ID());
				if (ps)
					UIProgressBarSatiety.SetProgressPos(s16(ps->experience_D*1000));
			}			
		};
		UIProgressBarPsyHealth.SetProgressPos(s16(pEntityAlive->conditions().GetPsyHealth()*1000));
		UIProgressBarRadiation.SetProgressPos(s16(pEntityAlive->conditions().GetRadiation()*1000));

/*
//.
		//  drag&drop    
		for(u32 i = 0; i <m_vDragDropItems.size(); i++) 
		{
			CInventoryItem* pItem = (CInventoryItem*)m_vDragDropItems[i]->GetData();
			if(pItem && !pItem->Useful())
			{
				if( m_vDragDropItems[i]->GetParent() )
					m_vDragDropItems[i]->GetParent()->DetachChild(m_vDragDropItems[i]);
				m_vDragDropItems[i]->SetData(NULL);
				m_vDragDropItems[i]->SetCustomDraw(NULL);
				
				if(m_pCurrentItem == pItem)
				{	
					SetCurrentItem(NULL);
					m_pCurrentDragDropItem = NULL;
				}

				xr_delete(m_vDragDropItems[i]);
				m_vDragDropItems.erase(m_vDragDropItems.begin()+i);
			}
		}
*/
		// update money
		CInventoryOwner* pOurInvOwner	= smart_cast<CInventoryOwner*>(pEntityAlive);
		string64						sMoney;
		int  iMoney						(pOurInvOwner->m_dwMoney);
		sprintf							(sMoney,"%d RU", iMoney);
		UIMoneyWnd.SetText				(sMoney);

		// update outfit parameters
		CCustomOutfit* outfit			= (CCustomOutfit*)pOurInvOwner->inventory().m_slots[OUTFIT_SLOT].m_pIItem;		
		UIOutfitInfo.Update				(*outfit);		
	}

	UITimeWnd.Update					();
	CUIWindow::Update					();
}

void CUIInventoryWnd::Show() 
{ 
	InitInventory			();
	inherited::Show			();

	if (!IsGameTypeSingle())
	{
		CActor *pActor = smart_cast<CActor*>(Level().CurrentEntity());
		if(!pActor) return;

		pActor->SetWeaponHideState(whs_INVENTORY_MENU, TRUE);
	}

	SendInfoToActor						("ui_inventory");

	Update								();
	PlaySnd								(eInvSndOpen);
}

void CUIInventoryWnd::Hide()
{
	PlaySnd								(eInvSndClose);
	inherited::Hide						();

	SendInfoToActor						("ui_inventory_hide");


	//    
	CActor *pActor = smart_cast<CActor*>(Level().CurrentEntity());
	if(pActor && m_iCurrentActiveSlot != NO_ACTIVE_SLOT && 
		pActor->inventory().m_slots[m_iCurrentActiveSlot].m_pIItem)
	{
		pActor->inventory().Activate(m_iCurrentActiveSlot);
		m_iCurrentActiveSlot = NO_ACTIVE_SLOT;
	}

	if (!IsGameTypeSingle())
	{
		CActor *pActor		= smart_cast<CActor*>(Level().CurrentEntity());
		if(!pActor)			return;

		pActor->SetWeaponHideState(whs_INVENTORY_MENU, FALSE);
	}
}

void CUIInventoryWnd::AttachAddon(PIItem item_to_upgrade)
{
	PlaySnd										(eInvAttachAddon);
	R_ASSERT									(item_to_upgrade);
	if (OnClient())
	{
		NET_Packet								P;
		item_to_upgrade->object().u_EventGen	(P, GE_ADDON_ATTACH, item_to_upgrade->object().ID());
		P.w_u32									(item_to_upgrade->object().ID());
		item_to_upgrade->object().u_EventSend	(P);
	};

	item_to_upgrade->Attach						(CurrentIItem());


	//          
	CActor *pActor								= smart_cast<CActor*>(Level().CurrentEntity());
	if(pActor && item_to_upgrade == pActor->inventory().ActiveItem())
	{
			m_iCurrentActiveSlot				= pActor->inventory().GetActiveSlot();
			pActor->inventory().Activate		(NO_ACTIVE_SLOT);
	}

//.	(smart_cast<CUIDragDropList*>(m_pCurrentDragDropItem->GetParent()))->DetachChild(m_pCurrentDragDropItem);
//.	m_pCurrentDragDropItem			= NULL;
	SetCurrentItem								(NULL);
}

void CUIInventoryWnd::DetachAddon(const char* addon_name)
{
	PlaySnd										(eInvDetachAddon);
	if (OnClient())
	{
		NET_Packet								P;
		CurrentIItem()->object().u_EventGen		(P, GE_ADDON_DETACH, CurrentIItem()->object().ID());
		P.w_stringZ								(addon_name);
		CurrentIItem()->object().u_EventSend	(P);
	};
	CurrentIItem()->Detach						(addon_name);

	//          
	CActor *pActor								= smart_cast<CActor*>(Level().CurrentEntity());
	if(pActor && CurrentIItem() == pActor->inventory().ActiveItem())
	{
			m_iCurrentActiveSlot				= pActor->inventory().GetActiveSlot();
			pActor->inventory().Activate		(NO_ACTIVE_SLOT);
	}
}


void	CUIInventoryWnd::SendEvent_ActivateSlot	(PIItem	pItem)
{
	NET_Packet						P;
	pItem->object().u_EventGen		(P, GEG_PLAYER_ACTIVATE_SLOT, pItem->object().H_Parent()->ID());
	P.w_u32							(pItem->GetSlot());
	pItem->object().u_EventSend		(P);
}

void	CUIInventoryWnd::SendEvent_Item2Slot			(PIItem	pItem)
{
	NET_Packet						P;
	pItem->object().u_EventGen		(P, GEG_PLAYER_ITEM2SLOT, pItem->object().H_Parent()->ID());
	P.w_u16							(pItem->object().ID());
	pItem->object().u_EventSend		(P);
	g_pInvWnd->PlaySnd				(eInvItemToSlot);
};

void	CUIInventoryWnd::SendEvent_Item2Belt			(PIItem	pItem)
{
	NET_Packet						P;
	pItem->object().u_EventGen		(P, GEG_PLAYER_ITEM2BELT, pItem->object().H_Parent()->ID());
	P.w_u16							(pItem->object().ID());
	pItem->object().u_EventSend		(P);
	g_pInvWnd->PlaySnd				(eInvItemToBelt);
};

void	CUIInventoryWnd::SendEvent_Item2Ruck			(PIItem	pItem)
{
	NET_Packet						P;
	pItem->object().u_EventGen		(P, GEG_PLAYER_ITEM2RUCK, pItem->object().H_Parent()->ID());
	P.w_u16							(pItem->object().ID());
	pItem->object().u_EventSend		(P);

	g_pInvWnd->PlaySnd				(eInvItemToRuck);
};

void	CUIInventoryWnd::SendEvent_Item_Sell			(PIItem	pItem)
{
	NET_Packet						P;
	pItem->object().u_EventGen		(P, GEG_PLAYER_ITEM_SELL, pItem->object().H_Parent()->ID());
	P.w_u16							(1);
	P.w_u16							(pItem->object().ID());
	pItem->object().u_EventSend		(P);

	g_pInvWnd->PlaySnd				(eInvDropItem);
};

void	CUIInventoryWnd::SendEvent_Item_Drop			(PIItem	pItem)
{
	NET_Packet						P;
	pItem->object().u_EventGen		(P, GEG_PLAYER_ITEMDROP, pItem->object().H_Parent()->ID());
	P.w_u16							(pItem->object().ID());
	pItem->object().u_EventSend		(P);

	g_pInvWnd->PlaySnd				(eInvDropItem);
};

void	CUIInventoryWnd::SendEvent_Item_Eat			(PIItem	pItem)
{
	NET_Packet						P;
	pItem->object().u_EventGen		(P, GEG_PLAYER_ITEM_EAT, pItem->object().H_Parent()->ID());
	P.w_u16							(pItem->object().ID());
	pItem->object().u_EventSend		(P);
};

void	CUIInventoryWnd::SendEvent_ActivateArtefact			(PIItem	pItem)
{
	NET_Packet						P;
	pItem->object().u_EventGen		(P, GEG_PLAYER_ACTIVATEARTEFACT, pItem->object().H_Parent()->ID());
	P.w_u16							(pItem->object().ID());
	pItem->object().u_EventSend		(P);	
};

void CUIInventoryWnd::BindDragDropListEnents(CUIDragDropListEx* lst)
{
	lst->m_f_item_drop				= CUIDragDropListEx::DRAG_DROP_EVENT(this,&CUIInventoryWnd::OnItemDrop);
	lst->m_f_item_start_drag		= CUIDragDropListEx::DRAG_DROP_EVENT(this,&CUIInventoryWnd::OnItemStartDrag);
	lst->m_f_item_db_click			= CUIDragDropListEx::DRAG_DROP_EVENT(this,&CUIInventoryWnd::OnItemDbClick);
	lst->m_f_item_selected			= CUIDragDropListEx::DRAG_DROP_EVENT(this,&CUIInventoryWnd::OnItemSelected);
	lst->m_f_item_rbutton_click		= CUIDragDropListEx::DRAG_DROP_EVENT(this,&CUIInventoryWnd::OnItemRButtonClick);
}


#include "../xr_level_controller.h"
#include <dinput.h>

bool CUIInventoryWnd::OnKeyboard(int dik, EUIMessages keyboard_action)
{
	if (UIPropertiesBox.GetVisible())
		UIPropertiesBox.OnKeyboard(dik, keyboard_action);

	if(dik==DIK_NUMPAD7 && keyboard_action==WINDOW_KEY_PRESSED && CurrentIItem()){
		CurrentIItem()->ChangeCondition(-0.05f);
		UIItemInfo.InitItem(CurrentIItem());
	}

	if(dik==DIK_NUMPAD8 && keyboard_action==WINDOW_KEY_PRESSED && CurrentIItem()){
		CurrentIItem()->ChangeCondition(0.05f);
		UIItemInfo.InitItem(CurrentIItem());
	}

	if( inherited::OnKeyboard(dik,keyboard_action) )return true;

	return false;
}
